package bus

import (
    "encoding/csv"
    "fmt"
    "io"
    "net/http"
    "path/filepath"
    "strconv"
    "strings"

    "github.com/tgckpg/golifehk/utils"
)

type BusStop struct {
    RouteId string
    Direction string
    StationSeq string
    StationId string
    Latitude float64
    Longtitude float64
    Name_zhant string
    Name_en string
}

var mBusStops *map[string]BusStop

var CSV_BUSSTOPS string = filepath.Join( utils.WORKDIR, "mtr_bus_stops.csv" )

func readBusStopData( r io.Reader ) ( *map[string]BusStop, error ) {

    reader := csv.NewReader( r )
    entries, err := reader.ReadAll()
    if err != nil {
        return nil, err
    }

    busStops := map[string]BusStop{}
    var headers []string
    for i, line := range entries {
        if i == 0 {
            headers = line
            line[0] = strings.TrimLeft( line[0], utils.BOM )
            continue
        }

        var entry BusStop
        for j, value := range line {
            switch headers[j] {
                case "ROUTE_ID":
                    entry.RouteId = value
                case "DIRECTION":
                    entry.Direction = value
                case "STATION_SEQNO":
                    entry.StationSeq = value
                case "STATION_ID":
                    entry.StationId = value
                case "STATION_LATITUDE":
                    v, _ := strconv.ParseFloat( value, 64 )
                    entry.Latitude = v
                case "STATION_LONGITUDE":
                    v, _ := strconv.ParseFloat( value, 64 )
                    entry.Longtitude = v
                case "STATION_NAME_CHI":
                    entry.Name_zhant = value
                case "STATION_NAME_ENG":
                    entry.Name_en = value
                default:
                    return nil, fmt.Errorf( "Unknown header \"%s\"", headers[j] )
            }
        }

        if _, t := busStops[ entry.StationId ]; t {
            return nil, fmt.Errorf( "Duplicated entry %+v", entry )
        }

        busStops[ entry.StationId ] = entry
    }
    return &busStops, nil
}

func getBusStops() (*map[string]BusStop, error) {

    QUERY_FUNC := func() ( io.ReadCloser, error ) {
        resp, err := http.Get( "https://opendata.mtr.com.hk/data/mtr_bus_stops.csv" )
        if err != nil {
            return nil, err
        }
        return resp.Body, nil
    }

    buff, err := utils.CacheStream( CSV_BUSSTOPS, QUERY_FUNC, 7 * 24 * 3600 )
    if err != nil {
        return nil, err
    }

    return readBusStopData( buff )
}
